/*
 *
 *  You can modify and use this source freely
 *  only for the development of application related Live2D.
 *
 *  (c) Live2D Inc. All rights reserved.
 */
#import "ViewController.h"
#import "IPhoneUtil.h"
#import "LAppModel.h"
#import "LAppDefine.h"
#include "LAppLive2DManager.h"


@implementation ViewController
{
	LAppLive2DManager* live2DMgr;
}


/*
 * 画面がロードされたときの処理。
 * @UIViewController
 */
- (void)viewDidLoad
{
    [super viewDidLoad];
	
	// Live2Dを初期化
    live2DMgr=new LAppLive2DManager();
	
	// OpenGL用Viewの作成。
    // OpenGL初期化前にテクスチャの読み込みなどのAPIを呼ばないこと。
    // また、メインスレッドで作成したコンテキストは他のスレッドからはアクセスできないので注意。
	CGRect screen = [IPhoneUtil getScreenRect];// 画面サイズを取得
    LAppView* view = live2DMgr->createView(screen);
	
	live2DMgr->changeModel();
	
	// 画面に表示
	[self.view addSubview:(UIView*)view] ;
	
	// モデル切り替え用のUIの追加
	MenuView *target = [[MenuView alloc] initWithFrame:CGRectMake(20, screen.size.height-50, 50, 50)] ;
	[target setParentController:self];
	[self.view addSubview:target];
	[target becomeFirstResponder];
}


/*
 * 画面が削除されるときの処理
 * @UIViewController
 */
- (void)viewDidUnload
{
	[super viewDidUnload];
    // Release any retained subviews of the main view.
    if (LAppDefine::DEBUG_LOG) NSLog(@"viewDidUnload @ViewController");
	delete live2DMgr;
	live2DMgr=nil;
}


/*
 * 非表示になる時の処理。
 * 不要なアニメーションを止める。
 * @UIViewController
 */
- (void)viewWillDisappear:(BOOL)animated
{
    if (LAppDefine::DEBUG_LOG) NSLog(@"viewWillDisappear @ViewController");
    live2DMgr->onPause();
    [super viewWillDisappear:animated];
}


/*
 * 表示されるときの処理。
 * 止めていたアニメーションを開始する
 * @UIViewController
 */
- (void)viewWillAppear:(BOOL)animated
{
    if (LAppDefine::DEBUG_LOG) NSLog(@"viewWillAppear @ViewController");
	live2DMgr->onResume();
    [super viewWillAppear:animated];
}


/*
 * メニューから選択したときのイベント
 */
-(void)event
{
	if (LAppDefine::DEBUG_LOG)NSLog(@"change model");
	live2DMgr->changeModel();
}

@end


/*
 * ポップアップメニューのUIView
 */
@implementation MenuView

/*
 * 初期化時
 * 画像を表示する
 */
- (id)initWithFrame:(CGRect)frame
{
    if ((self = [super initWithFrame:frame]))
	{
		UIImageView* image=[[UIImageView alloc]initWithImage:[UIImage imageNamed:@"icon_gear.png"]];
		[self addSubview:image];
    }
    return self;
}


/*
 * タッチ時。
 * メニューを作成する
 */
- (void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event
{
	UIMenuController *menu = [UIMenuController sharedMenuController];
	[menu setTargetRect:CGRectZero inView:self];
	menu.arrowDirection = UIMenuControllerArrowRight;
	
	UIMenuItem *item1 = [[UIMenuItem alloc] initWithTitle:@"change model" action:@selector(event:)] ;

	menu.menuItems = [NSArray arrayWithObjects:item1, nil];
	[menu setMenuVisible:YES animated:YES];
}


- (BOOL)canPerformAction:(SEL)action withSender:(id)sender
{
	if ( action == @selector(event:))
	{
		return YES;
	}
	return NO;
}


- (void)event:(id)sender
{
	[parentController event];
}


-(void)setParentController:(ViewController*)parent
{
	parentController=parent;
}


- (BOOL)canBecomeFirstResponder
{
	return YES;
}

@end

